-module(id_server2).

-behaviour(gen_server).

%% API
-export([start_link/0,
         get_id/0,
         stop/0]).

%% gen_server callbacks
-export([init/1, handle_call/3, handle_cast/2, handle_info/2, code_change/3,
         terminate/2]).

-define(SERVER, ?MODULE).
-define(TIMEOUT, 5000). % 5 seconds

-record(state, {id = 0}).

%%%=============================================================================
%%% API functions
%%%=============================================================================

start_link() ->
    gen_server:start_link({local, ?SERVER}, ?MODULE, [], [{timeout, ?TIMEOUT}]).

get_id() ->
    gen_server:call(?SERVER, get_id).

stop() ->
    gen_server:cast(?SERVER, stop).

%%%=============================================================================
%%% gen_server callbacks
%%%=============================================================================

init([]) ->
    {ok, #state{}}.

handle_call(get_id, _From, #state{id = NextId} = State) ->
    {reply, NextId, State#state{id = NextId + 1}};
handle_call(_Request, _From, State) ->
    {reply, ok, State}.

handle_cast(stop, State) ->
    {stop, normal, State};
handle_cast(_Request, State) ->
    {noreply, State}.

handle_info(_Info, State) ->
    {noreply, State}.

code_change(_OldVsn, State, _Extra) ->
    {ok, State}.

terminate(_Reason, _State) ->
    ok.
